from sqlite3 import connect, OperationalError
from arquee.config import logs_folder, execution_id, logs_file
from os.path import join


class ServerDB:
    def __init__(self):
        self.db_path = join(logs_folder, "arquee.db")

        conn = connect(self.db_path)
        cursor = conn.cursor()
        query = """ 
        CREATE TABLE IF NOT EXISTS history (
            id INTEGER,
            status TEXT DEFAULT 'RUNNING',
            timestamp DATETIME DEFAULT (datetime('now','localtime')),
            file_location TEXT)
        """

        cursor.execute(query)
        conn.commit()
        conn.close()

    def execute_query(self, query: str):
        conn = connect(self.db_path)
        cursor = conn.cursor()
        cursor.execute(query)
        conn.commit()
        conn.close()

    def create_row(self):
        query = f"INSERT INTO history (id ,file_location) VALUES ({execution_id}, '{logs_file}')"
        self.execute_query(query)

    def failed_row(self):
        query = f"UPDATE history SET status = 'FAILED' WHERE id = {execution_id}"
        self.execute_query(query)

    def done_row(self):
        query = f"UPDATE history SET status = 'DONE' WHERE id = {execution_id} AND status != 'FAILED'"
        self.execute_query(query)



from os import makedirs
import logging
from .config import logs_folder, logs_file


try:
    makedirs(logs_folder)
except Exception:
    pass

FORMAT = "%(asctime)s - %(levelname)s - %(message)s"
logging.basicConfig(filename=logs_file, level=logging.INFO, format=FORMAT)
logger = logging.getLogger()



from os.path import exists
from os import makedirs
from shutil import rmtree
from pyspark.sql import SparkSession
from .logger import logger
from .server.server_db import ServerDB


server_db = ServerDB()
server_db.create_row()


def arquee_error(message: str):
    logger.error(message)
    print("ERROR, EXITED")
    server_db.failed_row()
    exit(1)


def create_folder(folder_name: str):
    try:
        if exists(folder_name):
            rmtree(folder_name)
            makedirs(folder_name)
        else:
            makedirs(folder_name)

    except Exception as e:
        print("e")
        arquee_error(e)


def get_tables(tables_file: str) -> list[str]:
    try:
        with open(tables_file, "r") as file:
            tables = file.readlines()
            return [table.strip() for table in tables]
    except FileNotFoundError:
        arquee_error(f"File {tables_file} not found")


def create_spark_session() -> SparkSession:
    spark: SparkSession = (
        SparkSession.builder.appName("ETL")
        .master("local[*]")
        .config("spark.executor.memory", "4g")
        .config("spark.sql.parquet.int96RebaseModeInWrite", "CORRECTED")
        .getOrCreate()
    )
    return spark



import subprocess
import sys
from os.path import exists, normpath, join
from os import sep


def start_server():
    print(sys.executable)
    cmd = [
        sys.executable,
        "-m",
        "waitress",
        "serve",
        "arquee.server.web_server:app",
    ]

    with open("server.log", "wb"):
        subprocess.Popen(cmd)

    print("Servidor iniciado en segundo plano en el puerto 8000.")


def create_files():
    origin_example = 'ORIGIN_DB={"host": "host", "port":1433 , "user": "user", "pwd": "password", "db_name": "some_db"}\n'
    destiny_example = 'DESTINY_DB={"host": "host", "port":1433 , "user": "user", "pwd": "password", "db_name": "some_db"}\n'

    if not exists(".env"):
        with open(".env", "a") as file:
            file.write(origin_example)
            file.write(destiny_example)
            print("File .env created successfully")

    if not exists("tables.txt"):
        with open("tables.txt", "a") as file:
            file.write(
                "Here you can specify the tables for create the datalake, delete this line :)"
            )
            print("File tables.txt created successfully")


if __name__ == "__main__":
    create_files()



from json import loads, JSONDecodeError
from os import getenv
from sqlalchemy import create_engine
from sqlalchemy.exc import OperationalError
from .tools import arquee_error


class DB:
    def __init__(self, variable_name: str):
        self.url: str
        self.db_name: str
        self.db_host: str

        env_variable = getenv(variable_name)

        if env_variable:
            try:
                db: DB = loads(env_variable)
            except JSONDecodeError:
                arquee_error(f"Error decoding ->  {variable_name} variable")

        else:
            arquee_error(f"Missing variable : {variable_name} ")

        required_keys: set = {"host", "db_name", "user", "pwd", "port"}
        if not required_keys.issubset(db.keys()):
            arquee_error(f"missing keys {required_keys - db.keys()}")

        host = db["host"]
        db_name = db["db_name"]
        user = db["user"]
        pwd = db["pwd"]
        port = db["port"]

        test_connection_url = f"mssql+pymssql://{user}:{pwd}@{host}/?charset=utf8"
        try:
            create_engine(test_connection_url).connect()
        except OperationalError:
            arquee_error(f"Login failed with {variable_name} credentials")
        self.db_host = host
        self.db_name = db_name
        self.url = f"jdbc:sqlserver://{host}:{port};databaseName={db_name};user={user};password={pwd};encrypt=false;"



from .db import DB
from os.path import join
from .config import base_folder
from .tools import create_folder, get_tables, create_spark_session
from .logger import logger
from py4j.protocol import Py4JJavaError
from .tools import server_db


class Arquee:
    def __init__(
        self,
        origin_db: str = "ORIGIN_DB",
        destiny_db: str = "DESTINY_DB",
        tables_file: str = "tables.txt",
    ):
        self.origin_db = DB(origin_db)
        self.destiny_db = DB(destiny_db)

        logger.info(f"{origin_db} connected successfully")
        logger.info(f"{destiny_db} connected successfully")

        self.datalake = join(base_folder, "data", "datalake")
        self.warehouse = join(base_folder, "data", "warehouse")

        create_folder(self.datalake)
        create_folder(self.warehouse)

        logger.info("Datalake folder created successfully")
        logger.info("Warehouse folder created successfully")

        self.tables = get_tables(tables_file)

        logger.info(f"{tables_file} processed successfully")

        self.spark = None  # Inicializamos en None para evitar problemas de referencia

    def create_datalake(self):
        self.spark = create_spark_session()  # Guardamos la sesión de Spark

        for table in self.tables:
            try:
                table_path = join(self.datalake, table)
                df = self.spark.read.jdbc(self.origin_db.url, table)
                df.write.parquet(table_path, mode="overwrite")
                logger.info(f"Table {table} loaded successfully on datalake")
            except Py4JJavaError as e:
                logger.error(
                    f"{table} does not exist in the database {self.origin_db.db_name}"
                )
                print(e)
                continue

    def close(self):
        if self.spark:
            self.spark.stop()
            logger.info("Spark session closed")
        server_db.done_row()
        logger.info("Arquee instance closed properly")

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()



from os.path import join
from dotenv import load_dotenv
from time import time


load_dotenv()

execution_id = round(time())
base_folder = "arquee"
logs_folder = join(base_folder, "logs")
logs_file = join(logs_folder, f"{execution_id}.log")

